from .base_expression import BaseExpression

class EqualityExpression(BaseExpression):
    """An expression class for performing any equality checks

    They are used when filtering (greater than, less than, equals, etc.) but are also
    used for setting and updating attributes using the equals sign.
    """
    pass
